-- ICONS THAT SHOW TRINKET, RACE, ETC.
function ArenaLive_Icon_Initialise (self, addonName, frameType, iconID, texture, cooldown)
	
	-- Create a shortcut for the parent frame.
	local parent = self:GetParent();
	parent["icon"..iconID] = self;
	
	self.addonName = addonName;
	self.frameType = frameType;
	self.texture = texture;
	ArenaLive_Cooldown_Initialise(cooldown, self);
	
	
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:SetScript("OnEvent", ArenaLive_Icon_OnEvent);
end


function ArenaLive_Icon_UpdateUnit (self, unit)

	self.unit = unit;
	ArenaLive_Icon_Update(self)
end



function ArenaLive_Icon_Update (self)
	
	if (not self.unit) then
		return;
	end
	
	-- First we reset Cooldowns and variables in case the icon type has been changed.
	ArenaLive_Cooldown_Set(self.cooldown, GetTime(), 0)
	self.sharedSpellID = nil;
	self.sharedCD = nil;
	self.specID = nil;
	self.race = nil;
	self.gender = nil;
	self.class = nil;
	
	local id = self:GetID();
	local iconType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["icon"..id.."Type"];

	
	if ( iconType == "race" ) then
		ArenaLive_Icon_UpdateRace(self);
	elseif (iconType == "class" ) then
		ArenaLive_Icon_UpdateClass(self);
	elseif (iconType == "trinket" ) then
		ArenaLive_Icon_UpdateTrinket(self);
	elseif (iconType == "racial" ) then
		ArenaLive_Icon_UpdateRacial(self);
	elseif (iconType == "specialisation" ) then
		ArenaLive_Icon_UpdateSpec(self);
	elseif (iconType == "interruptOrDispel" ) then
		ArenaLive_Icon_UpdateInterruptOrDispel(self);
	elseif (iconType == "reaction" ) then
		ArenaLive_Icon_UpdateReaction(self);
	else
	
	end
end



function ArenaLive_Icon_OnEvent (self, event, ...)

	if (not self.unit) then
		return;
	end
	
	local id = self:GetID();
	local iconType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["icon"..id.."Type"];

	if ( event == "PLAYER_ENTERING_WORLD" ) and ( self.unit == "player" ) and ( iconType == "specialisation" ) then
	
		-- We only need this once at startup, because "PLAYER_SPECIALIZATION_CHANGED" fires too early to get SpecData when logging in.
		self:UnregisterEvent("PLAYER_ENTERING_WORLD");
		ArenaLive_Icon_UpdateSpec(self);
		
	elseif ( ( event == "PLAYER_SPECIALIZATION_CHANGED" ) and ( self.unit == "player" ) and ( iconType == "specialisation" ) ) then
		
		ArenaLive_Icon_UpdateSpec(self)
		
	elseif ( ( event == "UNIT_SPELLCAST_SUCCEEDED" ) and ( iconType == "trinket" or iconType == "racial" or iconType == "interruptOrDispel" ) ) then
	
		local unit = select(1, ...);
		local spellID = select(5, ...);
		
		if (unit == self.unit) then
		
				if ( spellID == self.spellID ) then
					ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.spellCD)
				elseif ( spellID == self.sharedSpellID ) then
					ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.sharedCD)
				end

		
		end
	end

end



function ArenaLive_Icon_UpdateRace (self)

	if ( not self.unit ) then
		return;
	end

	local _, race = UnitRace(self.unit);
	local gender = UnitSex(self.unit);
	local isPlayer = UnitIsPlayer(self.unit);
	if ( ( ( race ~= self.race ) or ( gender ~= self.gender ) ) and ( isPlayer ) ) then
		self.race = race;
		self.gender = gender;
		
		if (gender > 1 ) then
			self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CHARACTERCREATE-RACES");
		else
			self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Races2");
		end
		
		self.texture:SetTexCoord(unpack(RACE_GENDER_ICON_TCOORDS[race][gender]));
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
	end

end



function ArenaLive_Icon_UpdateClass (self)

	if ( not self.unit ) then
			return;
	end
	
	local _, class = UnitClass(self.unit);
	
	if  ( class ~= self.class ) and ( class ) then
		self.class = class;
		self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
		self.texture:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
	end	

end



function ArenaLive_Icon_UpdateTrinket (self)

	if ( not self.unit ) then
			return;
	end

	local _, faction = UnitFactionGroup(self.unit);
	local _, race = UnitRace(self.unit);
	local _, class = UnitClass(self.unit);
		
	if ( faction == "Alliance" ) then
		self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_01");
	else
		self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_02");
	end
		
	self.spellID =  ARENALIVE_SPELLS.trinket
	self.spellCD = 120;
		
	-- Some racials have a sharedCD with the PvP-Trinket, so we check if this is the case.
	if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
		if ( ARENALIVE_SPELLS.racials[race][class][3] > 0 ) then
			self.sharedSpellID = ARENALIVE_SPELLS.racials[race][class][1];
			self.sharedCD = ARENALIVE_SPELLS.racials[race][class][3];
		end
	else
		if ( ARENALIVE_SPELLS.racials[race][3] > 0 ) then
			self.sharedSpellID = ARENALIVE_SPELLS.racials[race][1];
			self.sharedCD = ARENALIVE_SPELLS.racials[race][3];		
		end
	end
	
	self.texture:SetTexCoord(0, 1, 0, 1);
end



function ArenaLive_Icon_UpdateRacial (self)

	if ( not self.unit ) then
			return;
	end

	local _, race = UnitRace(self.unit);
	local _, class = UnitClass(self.unit);
	local sharedCD;
		
	if (race == "Draenei") or (race == "Orc") or (race == "BloodElf") then
			self.spellID = ARENALIVE_SPELLS.racials[race][class][1];
			self.spellCD = ARENALIVE_SPELLS.racials[race][class][2];
			sharedCD = ARENALIVE_SPELLS.racials[race][class][3];
	else
			self.spellID = ARENALIVE_SPELLS.racials[race][1];
			self.spellCD = ARENALIVE_SPELLS.racials[race][2];
			sharedCD = ARENALIVE_SPELLS.racials[race][3];
	end
		
		
	local _, _, icon = GetSpellInfo(self.spellID);
	self.texture:SetTexture(icon);
		
	-- Again Some racials have a sharedCD...
	if (sharedCD > 0 ) then
		self.sharedSpellID = ARENALIVE_SPELLS.trinket;
		self.sharedCD = sharedCD;
	end
	
	self.texture:SetTexCoord(0, 1, 0, 1);
end



function ArenaLive_Icon_UpdateSpec (self)

	if ( not self.unit ) then
			return;
	end	
	local specID;
	
	-- Only for PlayerFrame, because we can get the specinfo without inspecting functions.
	if (self.unit == "player" ) then	
			
		specID = GetSpecialization();
			
		if ( specID ) then
				
			local _, _, _, icon = GetSpecializationInfo(specID);
			
			if ( icon ) then
				self.texture:SetTexture(icon);
				self.specIcon = icon;
			else
				self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			end
				
		else
			self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
		end
	end
		
	self.texture:SetTexCoord(0, 1, 0, 1);

end



function ArenaLive_Icon_UpdateInterruptOrDispel (self)

	if ( not self.unit ) then
			return;
	end	

	local _, class = UnitClass(self.unit);
	local isPlayer = UnitIsPlayer(self.unit);
	if ( class and isPlayer ) then
		local spellID, cooldown = unpack(ARENALIVE_SPELLS["dispelsOrInterrupts"][class]);
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);	
		self.spellID =  spellID
		self.spellCD = cooldown;

		local _, _, icon = GetSpellInfo(self.spellID);
		self.texture:SetTexture(icon);
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
		self.spellID =  nil;
		self.spellCD = nil;
	end
end



function ArenaLive_Icon_UpdateReaction (self)

	if ( not self.unit ) then
			return;
	end	

	local red, green, blue = UnitSelectionColor(self.unit);
	
	self.texture:SetTexture(red, green, blue, 1);

end