function ArenaLive_Options_OnLoad(self)

	-- Create a Panel in Blizzard AddOn Panel
	local version = ARENALIVE_DATABASE["options"]["version"];
	self.name = "ArenaLive";
	self.okay = ArenaLive_Options_Okay;
	self.cancel = ArenaLive_Options_Cancel;
	InterfaceOptions_AddCategory(self);
	self:RegisterEvent("ADDON_LOADED");
end

function ArenaLive_Options_OnEvent(self, event, ...)
	addonName = ...;
	if ( addonName == "ArenaLiveCore" ) then
		-- Option menu text:
		ArenaLive_OptionsPortraitOverlayTitle:SetText(ARENALIVE_LOCALISATION["ArenaLive_OptionsPortraitOverlayTitle"]);
		ArenaLive_OptionsPortraitOverlayDescription:SetText(ARENALIVE_LOCALISATION["ArenaLive_OptionsPortraitOverlayDescription"]);
		
		-- CheckButtons:
		ArenaLive_Options_CheckButtonInitialise(ArenaLive_OptionsDebug, "debug");
		ArenaLive_Options_CheckButtonInitialise(ArenaLive_OptionsShowCooldownText, "showCooldownText");
		
		-- Editboxes:
		ArenaLive_Options_EditBoxInitialise(ArenaLive_OptionsSecCastHistoryIconDelay, "castHistoryIconDuration")
		
		--  Portrait Overlay Sliders
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayDefCDSlider, "defCD");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayOffCDSlider, "burstCD");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayStunSlider, "stun");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlaySilenceSlider, "silence");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayCrowdControlSlider, "crowdControl");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayRootSlider, "root");
		ArenaLive_OptionsPortraitPrioSlider_Initialise(ArenaLive_OptionsPortraitOverlayDisarmSlider, "disarm");

		
	end
end



function ArenaLive_Options_Okay(self)
	
	-- CheckButtons:
	ARENALIVE_DATABASE["options"]["debug"] = ArenaLive_OptionsDebug:GetChecked();
	ARENALIVE_DATABASE["options"]["showCooldownText"] = ArenaLive_OptionsShowCooldownText:GetChecked();
	
	-- Editboxes:
	ARENALIVE_DATABASE["options"]["castHistoryIconDuration"] = tonumber(ArenaLive_OptionsSecCastHistoryIconDelay:GetText());
	
	--  Portrait Overlay Priorities
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["defCD"] = ArenaLive_OptionsPortraitOverlayDefCDSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["burstCD"] = ArenaLive_OptionsPortraitOverlayOffCDSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["stun"] = ArenaLive_OptionsPortraitOverlayStunSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["silence"] = ArenaLive_OptionsPortraitOverlaySilenceSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["crowdControl"] = ArenaLive_OptionsPortraitOverlayCrowdControlSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["root"] = ArenaLive_OptionsPortraitOverlayRootSlider:GetValue();
	ARENALIVE_DATABASE["options"]["overlayPriorities"]["disarm"] = ArenaLive_OptionsPortraitOverlayDisarmSlider:GetValue();
	
	ArenaLive_Database_UpdateSavedVariables(nil, ArenaLive_SavedVariables);
end



-- EDITBOXES
function ArenaLive_Options_EditBoxInitialise(self, boxType)

	local editBoxName = self:GetName();
	local textString = _G[editBoxName.."Text"];
	local editBoxText = textString:GetName();
	
	textString:SetText(ARENALIVE_LOCALISATION[editBoxText]);
	
	self:SetText(ARENALIVE_DATABASE["options"][boxType]);
	
	-- We need to reset the EditBox's cursor position, so that the text is shown correctly.
	self:SetCursorPosition(0);
	
end



-- CHECKBUTTONS
function ArenaLive_Options_CheckButtonInitialise(self, buttonType)
	local buttonName = self:GetName();
	local textString = _G[buttonName.."Text"];
	local buttonText = textString:GetName();
	
	textString:SetText(ARENALIVE_LOCALISATION[buttonText]);
	self:SetChecked(ARENALIVE_DATABASE["options"][buttonType]);
end



-- Slider
function ArenaLive_OptionsPortraitPrioSlider_Initialise(self, ccType)
	
	local prefix = self:GetName();
	local title = _G[prefix.."Title"];
	local currText = _G[prefix.."Curr"];
	local minText = _G[prefix.."Low"];
	local maxText = _G[prefix.."High"];
	local currValue = ARENALIVE_DATABASE["options"]["overlayPriorities"][ccType];
	local minValue, maxValue = self:GetMinMaxValues();
	
	title:SetText(ARENALIVE_LOCALISATION[title:GetName()]);
	minText:SetText(minValue);
	maxText:SetText(maxValue);
	currText:SetText(currValue);
	self:SetValue(currValue);

end



function ArenaLive_OptionsPortraitPrioSlider_OnValueChanged(self)
	
	local prefix = self:GetName();
	local currText = _G[prefix.."Curr"];
	local currValue = self:GetValue();
	
	currText:SetText(currValue);
end