--[[ Note: Some of ArenaLive's UnitFrames need a dynamic unitID. 
Therefore I've divided the frame loading process into two different functions.
The first initialises the frame structure, while the second one fills
the frame with unitdata and registers the events. ]]--
-- SetUnitID

-- OnEnter
-- OnLeave
-- UpdateTooltip

-- MAIN UNITFRAME FUNCTIONS
function ArenaLive_UnitFrame_Initialise (self, addonName, frameType, background, border, name, portrait, threeDPortrait, portraitOverlay, healthBar, healthBarText, powerBar, powerBarText, threatIndicator, threatFeedbackUnit, healPredictionBar)

	self.addonName = addonName;
	self.frameType = frameType;
	self.background = background;
	self.border = border;
	self.name = name;
	self.portrait = portrait;
	self.threeDPortrait = threeDPortrait;
	self.portraitOverlay = portraitOverlay;
	self.healthBar = healthBar;
	self.healthBarText = healthBarText;
	self.powerBar = powerBar;
	self.powerBarText = powerBarText;
	self.threatIndicator = threatIndicator;
	self.healPredictionBar = healPredictionBar;
	
	ArenaLive_HealthBar_Initialise(healthBar, healthBarText, addonName, frameType);
	ArenaLive_PowerBar_Initialise(powerBar, powerBarText, addonName, frameType);
	ArenaLiveHealPredictionBar_Initialise(self ,healPredictionBar, healthBar);
	ArenaLive_ThreatIndicator_Initialise(self, threatIndicator, threatFeedbackUnit);
	ArenaLive_PortraitOverlay_Initialise(portraitOverlay, self, portrait);
	
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_NAME_UPDATE");
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	self:RegisterEvent("UNIT_MODEL_CHANGED");
	self:RegisterEvent("UNIT_AURA");
	
	-- We use this to change Attributes, as soon as we are out of combat, if a self.unit change occurs infight.
	self:RegisterEvent("PLAYER_REGEN_ENABLED");

	if ( self.healPredictionBar ) then
		self:RegisterEvent("UNIT_MAXHEALTH");
		self:RegisterEvent("UNIT_HEAL_PREDICTION");
	end	

	-- If a fullscreen frame (e.g. like the WorldMapFrame) is shown, the PlayerModel resets. OnShow fixes this. 
	self:SetScript("OnShow", ArenaLive_UnitFrame_OnShow);

end



function ArenaLive_UnitFrame_UpdateUnit (self, unit)
	
	self.unit = unit;
	-- It is not allowed to change Attributes InFight, so we need 
	if ( not InCombatLockdown() ) then
		self:SetAttribute("unit", unit);
		RegisterUnitWatch(self);
	else
		self.needAttributeUpdate = true;	
	end
	
	if ( self.portraitOverlay ) then
		self.portraitOverlay:SetScript("OnUpdate", ArenaLive_PortraitOverlay_OnUpdate);
	end
	
	if ( self.healthBar ) then
		self.healthBar.unit = unit;
		self.healthBar:SetScript("OnEvent", ArenaLive_HealthBar_OnEvent);
		self.healthBar:SetScript("OnUpdate", ArenaLive_HealthBar_OnUpdate);
	end

	if ( self.powerBar ) then
		self.powerBar.unit = unit;
		self.powerBar:SetScript("OnEvent", ArenaLive_PowerBar_OnEvent);
		self.powerBar:SetScript("OnUpdate", ArenaLive_PowerBar_OnUpdate);
	end

	ArenaLive_UnitFrame_Update(self);
end


-- Reset function for dynamic unitID unit frames
function ArenaLive_UnitFrame_ResetFrame (self)

	self.unit = nil;
	if ( not InCombatLockdown() ) then
		UnregisterUnitWatch(self);
		self:SetAttribute("unit",nil);
	else
		self.needAttributeUpdate = true;
	end
	
	
	if ( self.healthBar ) then
		self.healthBar.unit = nil;
		self:SetScript("OnEvent", nil);
		self:SetScript("OnUpdate", nil);
		ArenaLive_HealthBar_Reset(healthBar);
	end
	
	if ( self.powerBar ) then
		self.powerBar.unit = unit;
		self:SetScript("OnEvent", nil);
		self:SetScript("OnUpdate", nil);
		ArenaLive_PowerBar_Reset(powerBar);
	end
	
	if ( self.threatIndicator ) then
		self.threatIndicator:Hide();
	end

end



function ArenaLive_UnitFrame_OnEvent (self, event, ...)

	if (not self.unit) then
		return;
	end
	
	local arg1 = select(1, ...)
	
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		-- For some reason, the player unitframe is shown as offline, when the frame is loaded on the initial login. I don't know why this is the case, but this way the problem is fixed.
		ArenaLive_UnitFrame_Update(self);
	elseif ( event == "UNIT_NAME_UPDATE" ) then
	elseif ( event == "UNIT_MODEL_CHANGED" and arg1 == self.unit ) then
		ArenaLive_UnitFrame_PortraitUpdate(self);
	elseif ( event == "UNIT_MAXHEALTH" ) then
		ArenaLiveHealPredictionBar_UpdateMax(self);
		ArenaLiveHealPredictionBar_Update(self);
	elseif ( event == "UNIT_HEAL_PREDICTION" ) then
		ArenaLiveHealPredictionBar_Update(self);
	elseif ( event == "UNIT_AURA" ) and ( arg1 == self.unit ) then
		ArenaLive_PortraitOverlay_QueryAuras(self, arg1);
	elseif (event == "UNIT_THREAT_SITUATION_UPDATE" and self.threatIndicator ) then
		ArenaLive_ThreatIndicator_Update(self, ...);
	elseif ( event == "PLAYER_REGEN_ENABLED" and self.needAttributeUpdate ) then
		-- Just a safetycheck for testing if CombatLockdown of frames really fades with this event.
		if ( not InCombatLockdown() ) then
			self.needAttributeUpdate = nil;
			self:SetAttribute("unit", self.unit);
			RegisterUnitWatch(self);
		end	
	end
	
end



function ArenaLive_UnitFrame_OnShow (self)

	local portraitType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["portraitType"];
	
	if ( ( portraitType == "3D" ) and ( self.unit ) ) then
		self.threeDPortrait:SetCamera(0);
	end
end



function ArenaLive_UnitFrame_Update (self)

	if (not self.unit) then
		return;
	end

	ArenaLive_UnitFrame_NameUpdate(self);
	ArenaLive_UnitFrame_PortraitUpdate(self);
	ArenaLive_HealthBar_Update(self.healthBar, self.unit)
	ArenaLive_PowerBar_Update(self.powerBar, self.unit)
	ArenaLiveHealPredictionBar_UpdateMax(self);
	ArenaLiveHealPredictionBar_Update(self);
	ArenaLive_ThreatIndicator_Update(self)

end



-- TO DO: NAMEHANDLER
function ArenaLive_UnitFrame_NameUpdate (self)

	if (not self.name) or (not self.unit) then
		return;
	end
	
	local colourMode = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["nameColourMode"];
	local isPlayer = UnitIsPlayer(self.unit);
	
	if ( colourMode == "byClass" and isPlayer ) then
	
		local _, class = UnitClass(self.unit);
		
		if ( class ) then
			self.name:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b);
		end
		
	elseif ( colourMode == "byReaction" or ( colourMode == "byClass" and not isPlayer ) ) then
	
		local red, green, blue = UnitSelectionColor(self.unit);
		self.name:SetTextColor(red, green, blue);
	else
		self.name:SetTextColor(1, 1, 1);
	end
	
	if ( self.overrideName ) then
		self.name:SetText(GetUnitName(self.overrideName));
	else
		self.name:SetText(GetUnitName(self.unit));
	end

end



-- PORTRAITHANDLER
function ArenaLive_UnitFrame_PortraitUpdate (self)

	if (not self.portrait) or (not self.unit) then
		return;
	end
	
	local frameName = self:GetName();
	local portraitType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["portraitType"];
	
	if ( portraitType == "3D" ) then
		self.portrait:SetTexture(0, 0, 0, 1);
		self.portrait:SetTexCoord(0, 1, 0, 1);
		self.threeDPortrait:Show();
		self.threeDPortrait:SetUnit(self.unit);
		self.threeDPortrait:SetCamera(0);
		
	elseif ( portraitType == "class" ) then
		
		local _, class = UnitClass(self.unit);
		local isPlayer = UnitIsPlayer(self.unit);
		if ( class and isPlayer ) then
			self.threeDPortrait:Hide();
			self.portrait:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
			self.portrait:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
		elseif ( class ) then
			-- If the unit is a NPC we fall back to 3D-Portrait.
			self.threeDPortrait:Show();
			self.portrait:SetTexture();
			self.threeDPortrait:SetUnit(self.unit);
			self.threeDPortrait:SetCamera(0);
		end
		
	end
end



function ArenaLive_UnitFrame_StartMoving(self, button)
	
	local isLocked = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["isLocked"];
	if ( not isLocked and button == "LeftButton" ) then
		self:StartMoving();
		self.isMoving = true;
		--self:SetUserPlaced(true);
		self:SetClampedToScreen(true);
	end
end



function ArenaLive_UnitFrame_StopMoving(self)
	if ( self.isMoving ) then
		self:StopMovingOrSizing();
		self.isMoving = false;
	end
end